import React from 'react';
import E from 'wangeditor';
import PreviewMenu from './PreviewMenu'
import Preview from './Preview'
import { fetch, apis } from '../../utils/request';

/**
 * @param {*} props
 * @returns
 */
class Wangedit extends React.Component {
  constructor(props, context) {
    super(props, context);
    this.state = {
      preview: false,
      html: undefined,
      contenteditable: props.contenteditable == false ? false : true
    }
    this.editor = undefined;
  }

  render() {
    return (
      <>
        <div ref="editorElem" style={{ textAlign: 'left' }} />
        <Preview width={426} style={{width: '426px', height: '863px', margin: 0, padding: 0}} visible={this.state.preview} html={this.state.html} onCancel={() => this.setState({preview: false})} />
      </>
    );
  }

  componentDidMount() {
    const elem = this.refs.editorElem
    this.editor = new E(elem)
    // 使用 onchange 函数监听内容的变化
    this.editor.config.onchange = html => {
      this.setState({ html })

      if (typeof this.props.onChange === 'function') {
        this.props.onChange(html)
      }
    }
    this.editor.config.zIndex = 100
    this.editor.config.uploadImgMaxLength = 1
    this.editor.config.customUploadImg = function (files, insert) {
      if (!files.length) return
      
      const data = new FormData()
      data.append('file', files[0])

      fetch(apis.image.upload)({data}).then(insert)
    }    

    // 扩展按钮
    this.editor.menus.extend('previewMenu', PreviewMenu)
    PreviewMenu.preview = (html) => {
      this.setState({preview: true})
    }

    this.editor.config.menus = [
      'head',  // 标题
      'bold',  // 粗体
      'fontSize',  // 字号
      'fontName',  // 字体
      'italic',  // 斜体
      'underline',  // 下划线
      'strikeThrough',  // 删除线
      'foreColor',  // 文字颜色
      'backColor',  // 背景颜色
      'list',  // 列表
      'justify',  // 对齐方式
      'quote',  // 引用
      'image',  // 插入图片
      'undo',  // 撤销
      'redo',  // 重复
      'previewMenu'
    ]
    
    // 过滤 word 字符
    this.editor.config.pasteFilterStyle = false
    this.editor.config.pasteTextHandle = function(content) {
      const regs = [
        /<!--\[if [\s\S]*?endif\]-->/ig,
        /<[a-zA-Z0-9]+\:[^>]+>[^>]*<\/[a-zA-Z0-9]+\:[^>]+>/ig,
        /<[a-zA-Z0-9]+\:[^>]+\/>/ig,
        /<style>[\s\S]*?<\/style>/ig,
        new RegExp('\u2029', 'ig'),     // 替换word分隔符 序号 8233
      ]

      return regs.reduce((acc, reg) => {
        return acc.replace(reg, '')
      }, content)
    }

    this.editor.create()
    this.editor.$textElem.attr('contenteditable',this.state.contenteditable);
    this.editor.config.uploadImgShowBase64 = true
    this.editor.txt.html(this.props.value)
  }

  componentDidUpdate(props, state) {
    if (this.props.value && !state.html) {
      if (this.editor) {
        this.editor.txt.html(this.props.value)
      }
    }
  }

  /**
   *增加这个 shouldComponentUpdate 生命函数
    处理自动聚焦到富文本上
   *
   * @param {*} nextProps
   * @returns
   * @memberof Wangedit
   */
  shouldComponentUpdate(nextProps) {
    return nextProps.value !== this.editor.txt.html() || nextProps.preview !== this.state.preview
  }
}

export default Wangedit

